@@ -32,6 +32,9 @@ pub trait VhostUserFrontend: VhostBackend {
     /// Query how many queues the backend supports.
     fn get_queue_num(&mut self) -> Result<u64>;
 
+    /// Disable all rings and reset the internal device state.
+    fn reset_device(&mut self) -> Result<()>;
+
     /// Signal backend to enable or disable corresponding vring.
     ///
     /// Backend must not pass data to/from the backend until ring is enabled by
@@ -72,6 +75,26 @@ pub trait VhostUserFrontend: VhostBackend {
 
     /// Remove a guest memory mapping from vhost.
     fn remove_mem_region(&mut self, region: &VhostUserMemoryRegionInfo) -> Result<()>;
+
+    /// Sends VHOST_USER_POSTCOPY_ADVISE msg to the backend
+    /// initiating the beginning of the postcopy process.
+    /// Backend will return a userfaultfd.
+    #[cfg(feature = "postcopy")]
+    fn postcopy_advise(&mut self) -> Result<File>;
+
+    /// Sends VHOST_USER_POSTCOPY_LISTEN msg to the backend
+    /// telling it to register its memory regions with
+    /// userfaultfd previously received through the
+    /// [`VhostUserFrontend::postcopy_advise`] call.
+    #[cfg(feature = "postcopy")]
+    fn postcopy_listen(&mut self) -> Result<()>;
+
+    /// Sends VHOST_USER_POSTCOPY_END msg to the backend
+    /// indicating the end of the postcopy process.
+    /// Backend will destroy the userfaultfd object previously
+    /// sent by [`VhostUserFrontend::postcopy_advise`].
+    #[cfg(feature = "postcopy")]
+    fn postcopy_end(&mut self) -> Result<()>;
 }
 
 fn error_code<T>(err: VhostUserError) -> Result<T> {
@@ -86,7 +109,7 @@ pub struct Frontend {
 
 impl Frontend {
     /// Create a new instance.
-    fn new(ep: Endpoint<FrontendReq>, max_queue_num: u64) -> Self {
+    fn new(ep: Endpoint<VhostUserMsgHeader<FrontendReq>>, max_queue_num: u64) -> Self {
         Frontend {
             node: Arc::new(Mutex::new(FrontendInternal {
                 main_sock: ep,
@@ -108,7 +131,10 @@ impl Frontend {
 
     /// Create a new instance from a Unix stream socket.
     pub fn from_stream(sock: UnixStream, max_queue_num: u64) -> Self {
-        Self::new(Endpoint::<FrontendReq>::from_stream(sock), max_queue_num)
+        Self::new(
+            Endpoint::<VhostUserMsgHeader<FrontendReq>>::from_stream(sock),
+            max_queue_num,
+        )
     }
 
     /// Create a new vhost-user frontend endpoint.
@@ -120,7 +146,7 @@ impl Frontend {
     pub fn connect<P: AsRef<Path>>(path: P, max_queue_num: u64) -> Result<Self> {
         let mut retry_count = 5;
         let endpoint = loop {
-            match Endpoint::<FrontendReq>::connect(&path) {
+            match Endpoint::<VhostUserMsgHeader<FrontendReq>>::connect(&path) {
                 Ok(endpoint) => break Ok(endpoint),
                 Err(e) => match &e {
                     VhostUserError::SocketConnect(why) => {
@@ -342,12 +368,9 @@ impl VhostUserFrontend for Frontend {
         let hdr = node.send_request_header(FrontendReq::GET_PROTOCOL_FEATURES, None)?;
         let val = node.recv_reply::<VhostUserU64>(&hdr)?;
         node.protocol_features = val.value;
-        // Should we support forward compatibility?
-        // If so just mask out unrecognized flags instead of return errors.
-        match VhostUserProtocolFeatures::from_bits(node.protocol_features) {
-            Some(val) => Ok(val),
-            None => error_code(VhostUserError::InvalidMessage),
-        }
+        Ok(VhostUserProtocolFeatures::from_bits_truncate(
+            node.protocol_features,
+        ))
     }
 
     fn set_protocol_features(&mut self, features: VhostUserProtocolFeatures) -> Result<()> {
@@ -375,6 +398,14 @@ impl VhostUserFrontend for Frontend {
         Ok(node.max_queue_num)
     }
 
+    fn reset_device(&mut self) -> Result<()> {
+        let mut node = self.node();
+        node.check_proto_feature(VhostUserProtocolFeatures::RESET_DEVICE)?;
+
+        let hdr = node.send_request_header(FrontendReq::RESET_DEVICE, None)?;
+        node.wait_for_ack(&hdr).map_err(|e| e.into())
+    }
+
     fn set_vring_enable(&mut self, queue_index: usize, enable: bool) -> Result<()> {
         let mut node = self.node();
         // set_vring_enable() is supported only when PROTOCOL_FEATURES has been enabled.
@@ -517,6 +548,36 @@ impl VhostUserFrontend for Frontend {
         let hdr = node.send_request_with_body(FrontendReq::REM_MEM_REG, &body, None)?;
         node.wait_for_ack(&hdr).map_err(|e| e.into())
     }
+
+    #[cfg(feature = "postcopy")]
+    fn postcopy_advise(&mut self) -> Result<File> {
+        let mut node = self.node();
+        node.check_proto_feature(VhostUserProtocolFeatures::PAGEFAULT)?;
+
+        let hdr = node.send_request_header(FrontendReq::POSTCOPY_ADVISE, None)?;
+        let (_, files) = node.recv_reply_with_files::<VhostUserEmpty>(&hdr)?;
+
+        match take_single_file(files) {
+            Some(file) => Ok(file),
+            None => error_code(VhostUserError::IncorrectFds),
+        }
+    }
+
+    #[cfg(feature = "postcopy")]
+    fn postcopy_listen(&mut self) -> Result<()> {
+        let mut node = self.node();
+        node.check_proto_feature(VhostUserProtocolFeatures::PAGEFAULT)?;
+        let hdr = node.send_request_header(FrontendReq::POSTCOPY_LISTEN, None)?;
+        node.wait_for_ack(&hdr).map_err(|e| e.into())
+    }
+
+    #[cfg(feature = "postcopy")]
+    fn postcopy_end(&mut self) -> Result<()> {
+        let mut node = self.node();
+        node.check_proto_feature(VhostUserProtocolFeatures::PAGEFAULT)?;
+        let hdr = node.send_request_header(FrontendReq::POSTCOPY_END, None)?;
+        node.wait_for_ack(&hdr).map_err(|e| e.into())
+    }
 }
 
 impl AsRawFd for Frontend {
@@ -550,7 +611,7 @@ impl VhostUserMemoryContext {
 
 struct FrontendInternal {
     // Used to send requests to the backend.
-    main_sock: Endpoint<FrontendReq>,
+    main_sock: Endpoint<VhostUserMsgHeader<FrontendReq>>,
     // Cached virtio features from the backend.
     virtio_features: u64,
     // Cached acked virtio features from the driver.
@@ -761,6 +822,8 @@ mod tests {
 
     use std::path::PathBuf;
 
+    const INVALID_PROTOCOL_FEATURE: u64 = 1 << 63;
+
     fn temp_path() -> PathBuf {
         PathBuf::from(format!(
             "/tmp/vhost_test_{}",
@@ -768,7 +831,9 @@ mod tests {
         ))
     }
 
-    fn create_pair<P: AsRef<Path>>(path: P) -> (Frontend, Endpoint<FrontendReq>) {
+    fn create_pair<P: AsRef<Path>>(
+        path: P,
+    ) -> (Frontend, Endpoint<VhostUserMsgHeader<FrontendReq>>) {
         let listener = Listener::new(&path, true).unwrap();
         listener.set_nonblocking(true).unwrap();
         let frontend = Frontend::connect(path, 2).unwrap();
@@ -783,7 +848,9 @@ mod tests {
         listener.set_nonblocking(true).unwrap();
 
         let frontend = Frontend::connect(&path, 1).unwrap();
-        let mut backend = Endpoint::<FrontendReq>::from_stream(listener.accept().unwrap().unwrap());
+        let mut backend = Endpoint::<VhostUserMsgHeader<FrontendReq>>::from_stream(
+            listener.accept().unwrap().unwrap(),
+        );
 
         assert!(frontend.as_raw_fd() > 0);
         // Send two messages continuously
@@ -885,7 +952,8 @@ mod tests {
 
         let pfeatures = VhostUserProtocolFeatures::all();
         let hdr = VhostUserMsgHeader::new(FrontendReq::GET_PROTOCOL_FEATURES, 0x4, 8);
-        let msg = VhostUserU64::new(pfeatures.bits());
+        // Unknown feature bits should be ignored.
+        let msg = VhostUserU64::new(pfeatures.bits() | INVALID_PROTOCOL_FEATURE);
         peer.send_message(&hdr, &msg, None).unwrap();
         let features = frontend.get_protocol_features().unwrap();
         assert_eq!(features, pfeatures);
@@ -951,7 +1019,7 @@ mod tests {
             .unwrap_err();
     }
 
-    fn create_pair2() -> (Frontend, Endpoint<FrontendReq>) {
+    fn create_pair2() -> (Frontend, Endpoint<VhostUserMsgHeader<FrontendReq>>) {
         let path = temp_path();
         let (frontend, peer) = create_pair(path);
 
