@@ -52,6 +52,9 @@ pub use self::backend_req_handler::{
 mod backend_req;
 #[cfg(feature = "vhost-user-backend")]
 pub use self::backend_req::Backend;
+mod gpu_backend_req;
+pub mod gpu_message;
+pub use self::gpu_backend_req::GpuBackend;
 
 /// Errors for vhost-user operations
 #[derive(Debug)]
@@ -218,6 +221,44 @@ pub(crate) fn take_single_file(files: Option<Vec<File>>) -> Option<File> {
     Some(files.swap_remove(0))
 }
 
+// Utility to generate `TryFrom` and `From` implementation for enums
+macro_rules! enum_value {
+    (
+        $(#[$meta:meta])*
+        $vis:vis enum $enum:ident: $T:tt {
+            $(
+                $(#[$variant_meta:meta])*
+                $variant:ident $(= $val:expr)?,
+            )*
+        }
+    ) => {
+        #[repr($T)]
+        $(#[$meta])*
+        $vis enum $enum {
+            $($(#[$variant_meta])* $variant $(= $val)?,)*
+        }
+
+        impl std::convert::TryFrom<$T> for $enum {
+            type Error = ();
+
+            fn try_from(v: $T) -> std::result::Result<Self, Self::Error> {
+                match v {
+                    $(v if v == $enum::$variant as $T => Ok($enum::$variant),)*
+                    _ => Err(()),
+                }
+            }
+        }
+
+        impl std::convert::From<$enum> for $T {
+            fn from(v: $enum) -> $T {
+                v as $T
+            }
+        }
+    }
+}
+
+use enum_value;
+
 #[cfg(all(test, feature = "vhost-user-backend"))]
 mod dummy_backend;
 
@@ -490,7 +531,7 @@ mod tests {
         frontend.set_vring_err(0, &eventfd).unwrap();
 
         let max_mem_slots = frontend.get_max_mem_slots().unwrap();
-        assert_eq!(max_mem_slots, 32);
+        assert_eq!(max_mem_slots, 509);
 
         let region_file: File = TempFile::new().unwrap().into_file();
         let region =
